﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 브러시 애니메이션
    /// </summary>
    public class BrushAnimation : AnimationTimeline
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Easing 함수 속성 - EasingFunctionProperty

        /// <summary>
        /// Easing 함수 속성
        /// </summary>
        public static readonly DependencyProperty EasingFunctionProperty = DependencyProperty.Register
        (
            "EasingFunction",
            typeof(IEasingFunction),
            typeof(BrushAnimation),
            new PropertyMetadata(null)
        );

        #endregion
        #region FROM 속성 - FromProperty

        /// <summary>
        /// FROM 속성
        /// </summary>
        public static readonly DependencyProperty FromProperty = DependencyProperty.Register
        (
            "From",
            typeof(Brush),
            typeof(BrushAnimation),
            new PropertyMetadata(null)
        );

        #endregion
        #region To 속성 - ToProperty

        /// <summary>
        /// To 속성
        /// </summary>
        public static readonly DependencyProperty ToProperty = DependencyProperty.Register
        (
            "To",
            typeof(Brush),
            typeof(BrushAnimation),
            new PropertyMetadata(null)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 타겟 속성 타입 - TargetPropertyType

        /// <summary>
        /// 타겟 속성 타입
        /// </summary>
        public override Type TargetPropertyType => typeof(Brush);

        #endregion
        #region Easing 함수 - EasingFunction

        /// <summary>
        /// Easing 함수
        /// </summary>
        public IEasingFunction EasingFunction
        {
            get
            {
                return (IEasingFunction)GetValue(EasingFunctionProperty);
            }
            set
            {
                SetValue(EasingFunctionProperty, value);
            }
        }

        #endregion
        #region From - From

        /// <summary>
        /// From
        /// </summary>
        public Brush From
        {
            get
            {
                return (Brush)GetValue(FromProperty);
            }
            set
            {
                SetValue(FromProperty, value);
            }
        }

        #endregion
        #region To - To

        /// <summary>
        /// To
        /// </summary>
        public Brush To
        {
            get
            {
                return (Brush)GetValue(ToProperty);
            }
            set
            {
                SetValue(ToProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 현재 값 구하기 - GetCurrentValue(defaultOriginValue, defaultDestinationValue, animationClock)

        /// <summary>
        /// 현재 값 구하기
        /// </summary>
        /// <param name="defaultOriginValue">디폴트 초기 값</param>
        /// <param name="defaultDestinationValue">디폴트 목적 값</param>
        /// <param name="animationClock">애니메이션 클럭</param>
        /// <returns>현재 값</returns>
        public override object GetCurrentValue(object defaultOriginValue, object defaultDestinationValue, AnimationClock animationClock)
        {
            if(!animationClock.CurrentProgress.HasValue)
            {
                return Brushes.Transparent;
            }

            Brush originValue      = From ?? defaultOriginValue      as Brush;
            Brush destinationValue = To   ?? defaultDestinationValue as Brush;

            double progress = animationClock.CurrentProgress.Value;

            if(progress == 0)
            {
                return originValue;
            }

            if(progress == 1)
            {
                return destinationValue;
            }

            IEasingFunction easingFunction = EasingFunction;

            if(easingFunction != null)
            {
                progress = easingFunction.Ease(progress);
            }

            return new VisualBrush
            (
                new Border()
                {
                    Width      = 1,
                    Height     = 1,
                    Background = originValue,
                    Child      = new Border()
                    {
                        Background = destinationValue,
                        Opacity    = progress,
                    }
                }
            );
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 인스턴스 생성하기 (코어) - CreateInstanceCore()

        /// <summary>
        /// 인스턴스 생성하기 (코어)
        /// </summary>
        /// <returns>인스턴스</returns>
        protected override Freezable CreateInstanceCore()
        {
            return new BrushAnimation();
        }

        #endregion
    }
}